"use strict";
/**
 * npm_install_proxy -- localhost NPM registry to `npm install` without network
 *
 * Copyright (C) 2020  SUSE LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
const http = require("http");
const fs = require("fs");
class Service {
    get url() { return this.baseUrl; }
    parseUrl(path) {
        if (!path)
            return undefined;
        const split = decodeURIComponent(path).split('/').filter(elem => typeof elem === 'string' && elem.length > 0);
        if (split.length >= 2 && split[0].startsWith("@")) {
            const scope = split.shift();
            split[0] = scope + '/' + split[0];
        }
        if (split[0] === '-') {
            split.shift();
            return {
                type: 'archive',
                package: split.join('/'),
            };
        }
        switch (split.length) {
            case 0:
                return {
                    type: 'root'
                };
            case 1:
                return {
                    type: 'pkg-versions',
                    package: split[0]
                };
            case 2:
                return {
                    type: 'package',
                    package: split[0],
                    version: split[1],
                };
        }
        return undefined;
    }
    run(registry) {
        this.server.on('request', (req, res) => {
            const headers = {
                'Content-Type': 'application/json'
            };
            const req_type = this.parseUrl(req.url);
            if (req.method !== 'GET') {
                console.log("request: (bad method)" + req.url);
                res.writeHead(500, "Invalid method", headers);
                res.end();
                return;
            }
            if (!req_type) {
                console.log("request (bad type): " + req.url);
                const error_message = "Invalid path";
                res.writeHead(400, error_message, headers);
                res.end();
                return;
            }
            try {
                switch (req_type.type) {
                    case 'root':
                        res.writeHead(200, headers);
                        res.end(JSON.stringify(registry.fetchPackages()));
                        break;
                    case 'package': {
                        const data = JSON.stringify(registry.fetchPkgVersion(req_type.package, req_type.version));
                        res.writeHead(200, headers);
                        res.end(data);
                        break;
                    }
                    case 'pkg-versions': {
                        const data = JSON.stringify(registry.fetchVersions(req_type.package));
                        res.writeHead(200, headers);
                        res.end(data);
                        break;
                    }
                    case 'archive': {
                        const archive_path = registry.archiveFile(req_type.package);
                        if (archive_path) {
                            res.writeHead(200, {
                                'Content-Type': 'application/x-compressed-tar'
                            });
                            fs.createReadStream(archive_path).pipe(res);
                        }
                        else {
                            console.log("request: (archive not found)" + req.url);
                            throw "not found";
                        }
                    }
                }
            }
            catch (_a) {
                console.log("request: (ERROR)" + req.url);
                res.writeHead(404);
                res.end("Not found");
            }
        });
        this.server.on('listening', () => {
            const addr = this.server.address();
            if (addr && typeof addr === 'object') {
                this.baseUrl.port = String(addr.port);
            }
        });
        const options = {
            host: this.baseUrl.hostname,
        };
        const port = Number(this.baseUrl.port);
        if (!Number.isNaN(port))
            options.port = port;
        return this.server.listen(options);
    }
    stop() {
        return new Promise((resolved, rejected) => {
            this.server.close(err => {
                if (err)
                    rejected(err);
                else
                    resolved();
            });
            return null;
        });
    }
    constructor(requestHandler) {
        if (!requestHandler.url)
            throw "Invalid URL: ";
        this.baseUrl = requestHandler.url;
        if (this.baseUrl.protocol !== "http:")
            throw Error("invalid protocol");
        const port = Number.parseInt(this.baseUrl.port);
        if (port < 1024)
            throw Error("invalid port");
        this.server = new http.Server();
    }
}
exports.Service = Service;
//# sourceMappingURL=service.js.map